"""``django-structlog`` is a structured logging integration for ``Django`` project using ``structlog``."""

name = "django_structlog"

VERSION = (10, 0, 0)

__version__ = ".".join(str(v) for v in VERSION)



"""
With these settings, tests run faster.
"""

import os

import environ
import structlog

env = environ.Env()

ROOT_DIR = (
    environ.Path(__file__) - 3
)  # (test_app/config/settings/base.py - 3 = test_app/)
APPS_DIR = ROOT_DIR.path("test_app")

# APPS
# ------------------------------------------------------------------------------
INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]


# GENERAL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#debug
DEBUG = False
# https://docs.djangoproject.com/en/dev/ref/settings/#use-tz
USE_TZ = True
# https://docs.djangoproject.com/en/dev/ref/settings/#secret-key
SECRET_KEY = env(
    "DJANGO_SECRET_KEY",
    default="SqlHVcvZwwazrUrjtUiMJerENM8bU3k2p7WZu1WgA4yc8R1DcDc2Rh54m8dRvWcs",
)
# https://docs.djangoproject.com/en/dev/ref/settings/#test-runner
TEST_RUNNER = "django.test.runner.DiscoverRunner"

# CACHES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#caches
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "",
    }
}

# EMAIL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#email-backend
EMAIL_BACKEND = "django.core.mail.backends.locmem.EmailBackend"
# https://docs.djangoproject.com/en/dev/ref/settings/#email-host
EMAIL_HOST = "localhost"
# https://docs.djangoproject.com/en/dev/ref/settings/#email-port
EMAIL_PORT = 1025

# Your stuff...
# ------------------------------------------------------------------------------

LOGGING = {
    "version": 1,
    "disable_existing_loggers": True,
    "formatters": {
        "plain": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": structlog.dev.ConsoleRenderer(colors=False),
        },
        "colored": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": structlog.dev.ConsoleRenderer(colors=True),
        },
    },
    "filters": {},
    "handlers": {
        "structured_stream": {"class": "logging.StreamHandler", "formatter": "colored"},
        "structured_file": {
            "class": "logging.handlers.WatchedFileHandler",
            "filename": "test.log",
            "formatter": "plain",
        },
    },
    "loggers": {"": {"handlers": ["structured_stream"], "level": "INFO"}},
}

structlog.configure(
    processors=[
        structlog.contextvars.merge_contextvars,
        structlog.stdlib.filter_by_level,
        structlog.processors.TimeStamper(fmt="iso"),
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
        structlog.processors.UnicodeDecoder(),
        structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
    ],
    logger_factory=structlog.stdlib.LoggerFactory(),
    cache_logger_on_first_use=True,
)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(str(ROOT_DIR), "db.sqlite3"),
    }
}

INSTALLED_APPS += ["django_structlog", "test_app"]

DJANGO_STRUCTLOG_COMMAND_LOGGING_ENABLED = True

IS_WORKER = False



"""
With these settings, tests run faster.
"""

# noinspection PyUnresolvedReferences
from .base import *  # noqa: F401,F403

# noinspection PyUnresolvedReferences
from .test import DATABASES, LOGGING  # noqa: F401

DJANGO_STRUCTLOG_COMMAND_LOGGING_ENABLED = True

IS_WORKER = False



import asyncio
import logging
import time

import structlog
from django.http import HttpResponse, StreamingHttpResponse

from django_structlog_demo_project.taskapp.celery import (
    failing_task,
    nesting_task,
    rejected_task,
    successful_task,
)

logger = structlog.get_logger(__name__)


def enqueue_successful_task(request):
    logger.info("Enqueuing successful task")
    successful_task.apply_async(foo="bar", priority=5)
    return HttpResponse(status=201)


def enqueue_failing_task(request):
    logger.info("Enqueuing failing task")
    failing_task.delay(foo="bar")
    return HttpResponse(status=201)


def enqueue_nesting_task(request):
    logger.info("Enqueuing nesting task")
    nesting_task.delay()
    return HttpResponse(status=201)


def log_with_standard_logger(request):
    logging.getLogger("foreign_logger").info("This is a standard logger")
    return HttpResponse(status=200)


def revoke_task(request):
    async_result = successful_task.apply_async(countdown=1)
    async_result.revoke()
    return HttpResponse(status=201)


def enqueue_unknown_task(request):
    from django_structlog_demo_project.taskapp.celery import unknown_task

    logger.info("Enqueuing unknown task")
    unknown_task.delay()
    return HttpResponse(status=201)


def enqueue_rejected_task(request):
    rejected_task.delay()
    return HttpResponse(status=201)


async def async_view(request):
    for num in range(1, 2):
        await asyncio.sleep(1)
        logger.info(f"This this is an async view {num}")
    return HttpResponse(status=200)


async def async_streaming_response():
    for chunk in range(0, 5):
        await asyncio.sleep(0.5)
        logger.info("streaming_chunk", chunk=chunk)
        yield chunk


def sync_streaming_response():
    for chunk in range(0, 5):
        time.sleep(0.5)
        logger.info("streaming_chunk", chunk=chunk)
        yield chunk


def sync_streaming_view(request):
    logger.info("This this is a sync streaming view")
    return StreamingHttpResponse(sync_streaming_response())


async def async_streaming_view(request):
    logger.info("This this is an async streaming view")
    return StreamingHttpResponse(async_streaming_response())


def raise_exception(request):
    raise Exception("This is a view raising an exception.")



import pytest
from ninja.testing import TestClient

from ..ninja_views import router

pytestmark = pytest.mark.django_db


class TestNinjaView:
    def test(self, caplog, request_factory):
        client = TestClient(router)
        response = client.get("/ninja")
        assert response.status_code == 200
        assert response.json() == {"result": "ok"}
        assert len(caplog.records) == 1
        record = caplog.records[0]
        assert record.msg["event"] == "This is a ninja structured log"



from django.contrib.auth.models import AbstractUser
from django.db.models import CharField
from django.urls import reverse
from django.utils.translation import gettext_lazy as _


class User(AbstractUser):
    # First Name and Last Name do not cover name patterns
    # around the globe.
    name = CharField(_("Name of User"), blank=True, max_length=255)

    def get_absolute_url(self):
        return reverse("users:detail", kwargs={"username": self.username})



import asyncio
import logging
import sys
import uuid
from typing import (
    TYPE_CHECKING,
    Any,
    AsyncGenerator,
    AsyncIterator,
    Awaitable,
    Callable,
    Generator,
    Iterator,
    Type,
    Union,
    cast,
)

import structlog
from asgiref import sync
from django.core.exceptions import PermissionDenied
from django.core.signals import got_request_exception
from django.http import Http404, StreamingHttpResponse
from django.utils.functional import SimpleLazyObject

from .. import signals
from ..app_settings import app_settings

if sys.version_info >= (3, 12, 0):
    from inspect import (  # type: ignore[attr-defined]
        iscoroutinefunction,
        markcoroutinefunction,
    )
else:
    from asgiref.sync import (  # type: ignore[no-redef]
        iscoroutinefunction,
        markcoroutinefunction,
    )

if TYPE_CHECKING:  # pragma: no cover
    from types import TracebackType

    from django.contrib.auth.base_user import AbstractBaseUser
    from django.http import HttpRequest, HttpResponse

logger = structlog.getLogger(__name__)


def get_request_header(request: "HttpRequest", header_key: str, meta_key: str) -> Any:
    if hasattr(request, "headers"):
        return request.headers.get(header_key)

    return request.META.get(meta_key)


def sync_streaming_content_wrapper(
    streaming_content: Iterator[bytes], context: Any
) -> Generator[bytes, None, None]:
    with structlog.contextvars.bound_contextvars(**context):
        logger.info("streaming_started")
        try:
            for chunk in streaming_content:
                yield chunk
        except Exception:
            logger.exception("streaming_failed")
            raise
        else:
            logger.info("streaming_finished")


async def async_streaming_content_wrapper(
    streaming_content: AsyncIterator[bytes], context: Any
) -> AsyncGenerator[bytes, Any]:
    with structlog.contextvars.bound_contextvars(**context):
        logger.info("streaming_started")
        try:
            async for chunk in streaming_content:
                yield chunk
        except asyncio.CancelledError:
            logger.warning("streaming_cancelled")
            raise
        except Exception:
            logger.exception("streaming_failed")
            raise
        else:
            logger.info("streaming_finished")


class RequestMiddleware:
    """``RequestMiddleware`` adds request metadata to ``structlog``'s logger context automatically.

    >>> MIDDLEWARE = [
    ...     # ...
    ...     'django_structlog.middlewares.RequestMiddleware',
    ... ]

    """

    sync_capable = True
    async_capable = True

    def __init__(
        self,
        get_response: Callable[
            ["HttpRequest"], Union["HttpResponse", Awaitable["HttpResponse"]]
        ],
    ) -> None:
        self.get_response = get_response
        if iscoroutinefunction(self.get_response):
            markcoroutinefunction(self)
        got_request_exception.connect(self.process_got_request_exception)

    def __call__(
        self, request: "HttpRequest"
    ) -> Union["HttpResponse", Awaitable["HttpResponse"]]:
        if iscoroutinefunction(self):
            return cast(RequestMiddleware, self).__acall__(request)
        self.prepare(request)
        response = cast("HttpResponse", self.get_response(request))
        self.handle_response(request, response)
        return response

    async def __acall__(self, request: "HttpRequest") -> "HttpResponse":
        await sync.sync_to_async(self.prepare)(request)
        try:
            response = await cast(Awaitable["HttpResponse"], self.get_response(request))
        except asyncio.CancelledError:
            logger.log(app_settings.REQUEST_CANCELLED_LOG_LEVEL, "request_cancelled")
            raise
        await sync.sync_to_async(self.handle_response)(request, response)
        return response

    def handle_response(self, request: "HttpRequest", response: "HttpResponse") -> None:
        if not hasattr(request, "_raised_exception"):
            self.bind_user_id(request)
            context = structlog.contextvars.get_merged_contextvars(logger)

            log_kwargs = dict(
                code=response.status_code,
                request=self.format_request(request),
            )
            signals.bind_extra_request_finished_metadata.send(
                sender=self.__class__,
                request=request,
                logger=logger,
                response=response,
                log_kwargs=log_kwargs,
            )
            if response.status_code >= 500:
                level = app_settings.STATUS_5XX_LOG_LEVEL
            elif response.status_code >= 400:
                level = app_settings.STATUS_4XX_LOG_LEVEL
            else:
                level = logging.INFO
            logger.log(
                level,
                "request_finished",
                **log_kwargs,
            )
            if isinstance(response, StreamingHttpResponse):
                streaming_content = response.streaming_content
                if response.is_async:
                    response.streaming_content = async_streaming_content_wrapper(
                        cast(AsyncIterator[bytes], streaming_content), context
                    )
                else:
                    response.streaming_content = sync_streaming_content_wrapper(
                        cast(Iterator[bytes], streaming_content), context
                    )

        else:
            exception = getattr(request, "_raised_exception")
            delattr(request, "_raised_exception")
            signals.update_failure_response.send(
                sender=self.__class__,
                request=request,
                response=response,
                logger=logger,
                exception=exception,
            )
        structlog.contextvars.clear_contextvars()

    def prepare(self, request: "HttpRequest") -> None:
        request_id = get_request_header(
            request, "x-request-id", "HTTP_X_REQUEST_ID"
        ) or str(uuid.uuid4())
        correlation_id = get_request_header(
            request, "x-correlation-id", "HTTP_X_CORRELATION_ID"
        )
        structlog.contextvars.bind_contextvars(request_id=request_id)
        self.bind_user_id(request)
        if correlation_id:
            structlog.contextvars.bind_contextvars(correlation_id=correlation_id)
        if app_settings.IP_LOGGING_ENABLED:
            self.bind_ip(request)
        log_kwargs = {
            "request": self.format_request(request),
            "user_agent": request.META.get("HTTP_USER_AGENT"),
        }
        signals.bind_extra_request_metadata.send(
            sender=self.__class__, request=request, logger=logger, log_kwargs=log_kwargs
        )
        logger.info("request_started", **log_kwargs)

    @classmethod
    def bind_ip(cls, request: "HttpRequest") -> None:
        from ipware import get_client_ip  # type: ignore[import-untyped]

        ip, _ = get_client_ip(request)
        structlog.contextvars.bind_contextvars(ip=ip)

    @staticmethod
    def format_request(request: "HttpRequest") -> str:
        return f"{request.method} {request.get_full_path()}"

    @staticmethod
    def bind_user_id(request: "HttpRequest") -> None:
        user_id_field = app_settings.USER_ID_FIELD
        if not user_id_field or not hasattr(request, "user"):
            return

        session_was_accessed = (
            request.session.accessed if hasattr(request, "session") else None
        )

        if request.user is not None:
            user_id = None
            if hasattr(request.user, user_id_field):
                user_id = getattr(request.user, user_id_field)
                if isinstance(user_id, uuid.UUID):
                    user_id = str(user_id)
            structlog.contextvars.bind_contextvars(user_id=user_id)

        if session_was_accessed is False:
            """using SessionMiddleware but user was never accessed, must reset accessed state"""
            user = request.user

            def get_user() -> Any:
                request.session.accessed = True
                return user

            request.user = cast("AbstractBaseUser", SimpleLazyObject(get_user))
            request.session.accessed = False

    def process_got_request_exception(
        self, sender: Type[Any], request: "HttpRequest", **kwargs: Any
    ) -> None:
        if not hasattr(request, "_raised_exception"):
            ex = cast(
                tuple[Type[Exception], Exception, "TracebackType"],
                sys.exc_info(),
            )
            self._process_exception(request, ex[1])

    def _process_exception(self, request: "HttpRequest", exception: Exception) -> None:
        if isinstance(exception, (Http404, PermissionDenied)):
            # We don't log an exception here, and we don't set that we handled
            # an error as we want the standard `request_finished` log message
            # to be emitted.
            return

        setattr(request, "_raised_exception", exception)
        self.bind_user_id(request)
        log_kwargs = dict(
            code=500,
            request=self.format_request(request),
        )
        signals.bind_extra_request_failed_metadata.send(
            sender=self.__class__,
            request=request,
            logger=logger,
            exception=exception,
            log_kwargs=log_kwargs,
        )
        logger.exception(
            "request_failed",
            **log_kwargs,
        )



import logging
import time
from signal import SIGTERM
from typing import Any, Optional, Type
from unittest.mock import MagicMock, Mock, call, patch

import structlog
from celery import shared_task
from django.contrib.auth.models import AnonymousUser
from django.dispatch import receiver as django_receiver
from django.test import RequestFactory, TestCase

from django_structlog.celery import receivers, signals


class TestReceivers(TestCase):
    def setUp(self) -> None:
        self.factory = RequestFactory()
        self.logger = structlog.getLogger(__name__)

    def tearDown(self) -> None:
        structlog.contextvars.clear_contextvars()

    def test_defer_task(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"

        request = self.factory.get("/foo")
        request.user = AnonymousUser()

        @shared_task
        def test_task(value: Any) -> None:  # pragma: no cover
            pass

        receiver = receivers.CeleryReceiver()
        receiver.connect_signals()
        structlog.contextvars.bind_contextvars(request_id=expected_uuid)
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            test_task.delay("foo")
        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_enqueued", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertIn("child_task_id", record.msg)
        self.assertEqual(expected_uuid, record.msg["request_id"])

    def test_receiver_before_task_publish_celery_protocol_v2(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"
        expected_user_id = "1234"
        expected_parent_task_uuid = "11111111-1111-1111-1111-111111111111"

        headers: dict[str, Any] = {}
        structlog.contextvars.bind_contextvars(
            request_id=expected_uuid,
            user_id=expected_user_id,
            task_id=expected_parent_task_uuid,
        )
        receiver = receivers.CeleryReceiver()
        receiver.receiver_before_task_publish(headers=headers)

        self.assertDictEqual(
            {
                "__django_structlog__": {
                    "request_id": expected_uuid,
                    "user_id": expected_user_id,
                    "parent_task_id": expected_parent_task_uuid,
                }
            },
            headers,
        )

    def test_receiver_before_task_publish_celery_protocol_v1(self) -> None:
        """Protocol v1 does not allow to store metadata"""
        expected_uuid = "00000000-0000-0000-0000-000000000000"
        expected_user_id = "1234"
        expected_parent_task_uuid = "11111111-1111-1111-1111-111111111111"

        headers: dict[str, Any] = {}
        structlog.contextvars.bind_contextvars(
            request_id=expected_uuid,
            user_id=expected_user_id,
            task_id=expected_parent_task_uuid,
        )
        mock_current_app = MagicMock()
        mock_conf = MagicMock()
        mock_current_app.conf = mock_conf
        mock_conf.task_protocol = 1
        receiver = receivers.CeleryReceiver()
        with patch("django_structlog.celery.receivers.current_app", mock_current_app):
            receiver.receiver_before_task_publish(headers=headers)

        self.assertDictEqual(
            {},
            headers,
        )

    def test_signal_modify_context_before_task_publish_celery_protocol_v2(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"
        user_id = "1234"
        expected_parent_task_uuid = "11111111-1111-1111-1111-111111111111"
        routing_key = "foo"
        properties: dict[str, Optional[str]] = {
            "correlation_id": "22222222-2222-2222-2222-222222222222"
        }

        received_properties: Any = None
        received_routing_key: Any = None

        @django_receiver(signals.modify_context_before_task_publish)
        def receiver_modify_context_before_task_publish(
            sender: Type[Any],
            signal: Any,
            context: Any,
            task_properties: Any,
            task_routing_key: str,
            **kwargs: Any,
        ) -> None:
            keys_to_keep = {"request_id", "parent_task_id"}
            new_dict = {
                key_to_keep: context[key_to_keep]
                for key_to_keep in keys_to_keep
                if key_to_keep in context
            }
            context.clear()
            context.update(new_dict)
            nonlocal received_properties
            received_properties = task_properties
            nonlocal received_routing_key
            received_routing_key = task_routing_key

        headers: dict[str, Any] = {}
        structlog.contextvars.bind_contextvars(
            request_id=expected_uuid,
            user_id=user_id,
            task_id=expected_parent_task_uuid,
        )
        receiver = receivers.CeleryReceiver()
        receiver.receiver_before_task_publish(
            headers=headers,
            routing_key=routing_key,
            properties=properties,
        )

        self.assertDictEqual(
            {
                "__django_structlog__": {
                    "request_id": expected_uuid,
                    "parent_task_id": expected_parent_task_uuid,
                }
            },
            headers,
            "Only `request_id` and `parent_task_id` are preserved",
        )
        self.assertDictEqual(
            {"correlation_id": "22222222-2222-2222-2222-222222222222"},
            received_properties,
        )
        self.assertEqual("foo", received_routing_key)

    def test_receiver_after_task_publish(self) -> None:
        expected_task_id = "00000000-0000-0000-0000-000000000000"
        expected_task_name = "Foo"
        headers: dict[str, Any] = {"id": expected_task_id, "task": expected_task_name}
        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            receiver.receiver_after_task_publish(headers=headers)

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_enqueued", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertIn("child_task_id", record.msg)
        self.assertEqual(expected_task_id, record.msg["child_task_id"])
        self.assertIn("child_task_name", record.msg)
        self.assertEqual(expected_task_name, record.msg["child_task_name"])

    def test_receiver_after_task_publish_protocol_v1(self) -> None:
        expected_task_id = "00000000-0000-0000-0000-000000000000"
        expected_task_name = "Foo"
        body: dict[str, Any] = {"id": expected_task_id, "task": expected_task_name}

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            receiver.receiver_after_task_publish(body=body)

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_enqueued", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertIn("child_task_id", record.msg)
        self.assertEqual(expected_task_id, record.msg["child_task_id"])
        self.assertIn("child_task_name", record.msg)
        self.assertEqual(expected_task_name, record.msg["child_task_name"])

    def test_receiver_task_pre_run(self) -> None:
        expected_request_uuid = "00000000-0000-0000-0000-000000000000"
        task_id = "11111111-1111-1111-1111-111111111111"
        expected_user_id = "1234"
        task = Mock()
        task.request = Mock()
        task.request.__django_structlog__ = {
            "request_id": expected_request_uuid,
            "user_id": expected_user_id,
        }
        task.name = "task_name"
        structlog.contextvars.bind_contextvars(foo="bar")

        context = structlog.contextvars.get_merged_contextvars(self.logger)
        self.assertDictEqual({"foo": "bar"}, context)
        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            receiver.receiver_task_prerun(task_id, task)
        context = structlog.contextvars.get_merged_contextvars(self.logger)

        self.assertDictEqual(
            {
                "task_id": task_id,
                "request_id": expected_request_uuid,
                "user_id": expected_user_id,
            },
            context,
        )

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_started", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertIn("task", record.msg)
        self.assertEqual("task_name", record.msg["task"])

    def test_signal_bind_extra_task_metadata(self) -> None:
        @django_receiver(signals.bind_extra_task_metadata)
        def receiver_bind_extra_request_metadata(
            sender: Type[Any], signal: Any, task: Any = None, logger: Any = None
        ) -> None:
            structlog.contextvars.bind_contextvars(
                correlation_id=task.request.correlation_id
            )

        expected_correlation_uuid = "00000000-0000-0000-0000-000000000000"
        task_id = "11111111-1111-1111-1111-111111111111"
        task = Mock()
        task.request = Mock()
        task.request.correlation_id = expected_correlation_uuid
        structlog.contextvars.bind_contextvars(foo="bar")

        context = structlog.contextvars.get_merged_contextvars(self.logger)
        self.assertDictEqual({"foo": "bar"}, context)
        receiver = receivers.CeleryReceiver()
        receiver.receiver_task_prerun(task_id, task)
        context = structlog.contextvars.get_merged_contextvars(self.logger)

        self.assertEqual(context["correlation_id"], expected_correlation_uuid)
        self.assertEqual(context["task_id"], task_id)

    def test_receiver_task_retry(self) -> None:
        expected_reason = "foo"

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.WARNING
        ) as log_results:
            receiver.receiver_task_retry(reason=expected_reason)

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_retrying", record.msg["event"])
        self.assertEqual("WARNING", record.levelname)
        self.assertIn("reason", record.msg)
        self.assertEqual(expected_reason, record.msg["reason"])

    def test_receiver_task_success(self) -> None:
        expected_result = "foo"

        @django_receiver(signals.pre_task_succeeded)
        def receiver_pre_task_succeeded(
            sender: Type[Any],
            signal: Any,
            task: Any = None,
            logger: Any = None,
            result: Any = None,
        ) -> None:
            structlog.contextvars.bind_contextvars(result=result)

        mock_sender = Mock()
        mock_sender.request._django_structlog_started_at = time.monotonic()

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            receiver.receiver_task_success(result=expected_result, sender=mock_sender)

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_succeeded", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertIn("result", record.msg)
        self.assertIn("duration_ms", record.msg)
        self.assertGreaterEqual(record.msg["duration_ms"], 0)
        self.assertEqual(expected_result, record.msg["result"])

    def test_receiver_task_success_no_started_at(self) -> None:
        expected_result = "foo"

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            receiver.receiver_task_success(result=expected_result)

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_succeeded", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertNotIn("duration_ms", record.msg)

    def test_receiver_task_failure(self) -> None:
        expected_exception = "foo"
        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.ERROR
        ) as log_results:
            receiver.receiver_task_failure(exception=Exception("foo"))

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_failed", record.msg["event"])
        self.assertEqual("ERROR", record.levelname)
        self.assertIn("error", record.msg)
        self.assertIn("exception", record.msg)
        self.assertEqual(expected_exception, record.msg["error"])

    def test_receiver_task_failure_with_throws(self) -> None:
        expected_exception = "foo"

        mock_sender = Mock()
        mock_sender.throws = (Exception,)
        mock_sender.request._django_structlog_started_at = time.monotonic()
        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            receiver.receiver_task_failure(
                exception=Exception("foo"), sender=mock_sender
            )

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_failed", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertIn("error", record.msg)
        self.assertIn("duration_ms", record.msg)
        self.assertNotIn("exception", record.msg)
        self.assertEqual(expected_exception, record.msg["error"])

    def test_receiver_task_revoked(self) -> None:
        expected_request_uuid = "00000000-0000-0000-0000-000000000000"
        task_id = "11111111-1111-1111-1111-111111111111"
        expected_user_id = "1234"
        expected_task_name = "task_name"
        request = Mock()
        request.__django_structlog__ = {
            "request_id": expected_request_uuid,
            "user_id": expected_user_id,
        }
        request.task = expected_task_name
        request.id = task_id

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.WARNING
        ) as log_results:
            receiver.receiver_task_revoked(
                request=request, terminated=False, signum=None, expired=False
            )

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_revoked", record.msg["event"])
        self.assertEqual("WARNING", record.levelname)
        self.assertIn("terminated", record.msg)
        self.assertFalse(record.msg["terminated"])
        self.assertIn("signum", record.msg)
        self.assertIsNone(record.msg["signum"])
        self.assertIn("signame", record.msg)
        self.assertIsNone(record.msg["signame"])
        self.assertIn("expired", record.msg)
        self.assertFalse(record.msg["expired"])
        self.assertIn("task_id", record.msg)
        self.assertEqual(task_id, record.msg["task_id"])
        self.assertIn("task", record.msg)
        self.assertEqual(expected_task_name, record.msg["task"])
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_request_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertEqual(expected_user_id, record.msg["user_id"])

    def test_receiver_task_revoked_terminated(self) -> None:
        expected_request_uuid = "00000000-0000-0000-0000-000000000000"
        task_id = "11111111-1111-1111-1111-111111111111"
        expected_user_id = "1234"
        expected_task_name = "task_name"
        request = Mock()
        request.__django_structlog__ = {
            "request_id": expected_request_uuid,
            "user_id": expected_user_id,
        }
        request.task = expected_task_name
        request.id = task_id

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.WARNING
        ) as log_results:
            receiver.receiver_task_revoked(
                request=request, terminated=True, signum=SIGTERM, expired=False
            )

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_revoked", record.msg["event"])
        self.assertEqual("WARNING", record.levelname)
        self.assertIn("terminated", record.msg)
        self.assertTrue(record.msg["terminated"])
        self.assertIn("signum", record.msg)
        self.assertEqual(15, record.msg["signum"])
        self.assertIn("signame", record.msg)
        self.assertEqual("SIGTERM", record.msg["signame"])
        self.assertIn("expired", record.msg)
        self.assertFalse(record.msg["expired"])
        self.assertIn("task_id", record.msg)
        self.assertEqual(task_id, record.msg["task_id"])
        self.assertIn("task", record.msg)
        self.assertEqual(expected_task_name, record.msg["task"])
        self.assertIn("request_id", record.msg)
        self.assertEqual(expected_request_uuid, record.msg["request_id"])
        self.assertIn("user_id", record.msg)
        self.assertEqual(expected_user_id, record.msg["user_id"])

    def test_receiver_task_unknown(self) -> None:
        task_id = "11111111-1111-1111-1111-111111111111"
        expected_task_name = "task_name"

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.ERROR
        ) as log_results:
            receiver.receiver_task_unknown(id=task_id, name=expected_task_name)

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_not_found", record.msg["event"])
        self.assertEqual("ERROR", record.levelname)
        self.assertIn("task_id", record.msg)
        self.assertEqual(task_id, record.msg["task_id"])
        self.assertIn("task", record.msg)
        self.assertEqual(expected_task_name, record.msg["task"])

    def test_receiver_task_rejected(self) -> None:
        task_id = "11111111-1111-1111-1111-111111111111"
        message = Mock(name="message")
        message.properties = dict(correlation_id=task_id)

        receiver = receivers.CeleryReceiver()
        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.ERROR
        ) as log_results:
            receiver.receiver_task_rejected(message=message)

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_rejected", record.msg["event"])
        self.assertEqual("ERROR", record.levelname)
        self.assertIn("task_id", record.msg)
        self.assertEqual(task_id, record.msg["task_id"])

    def test_priority(self) -> None:
        expected_uuid = "00000000-0000-0000-0000-000000000000"
        user_id = "1234"
        expected_parent_task_uuid = "11111111-1111-1111-1111-111111111111"
        expected_routing_key = "foo"
        expected_priority = 6
        properties = {"priority": expected_priority}

        headers: dict[str, Any] = {}
        structlog.contextvars.bind_contextvars(
            request_id=expected_uuid,
            user_id=user_id,
            task_id=expected_parent_task_uuid,
        )
        receiver = receivers.CeleryReceiver()
        receiver.receiver_before_task_publish(
            headers=headers,
            routing_key=expected_routing_key,
            properties=properties,
        )

        self.assertDictEqual(
            {
                "__django_structlog__": {
                    "user_id": user_id,
                    "request_id": expected_uuid,
                    "parent_task_id": expected_parent_task_uuid,
                }
            },
            headers,
            "Only `request_id` and `parent_task_id` are preserved",
        )

        expected_task_id = "00000000-0000-0000-0000-000000000000"
        expected_task_name = "Foo"
        headers = {"id": expected_task_id, "task": expected_task_name}

        with self.assertLogs(
            logging.getLogger("django_structlog.celery.receivers"), logging.INFO
        ) as log_results:
            receiver.receiver_after_task_publish(
                headers=headers, routing_key=expected_routing_key
            )

        self.assertEqual(1, len(log_results.records))
        record: Any = log_results.records[0]
        self.assertEqual("task_enqueued", record.msg["event"])
        self.assertEqual("INFO", record.levelname)
        self.assertIn("child_task_id", record.msg)
        self.assertEqual(expected_task_id, record.msg["child_task_id"])
        self.assertIn("child_task_name", record.msg)
        self.assertEqual(expected_task_name, record.msg["child_task_name"])

        self.assertIn("priority", record.msg)
        self.assertEqual(expected_priority, record.msg["priority"])

        self.assertIn("routing_key", record.msg)
        self.assertEqual(expected_routing_key, record.msg["routing_key"])


class TestConnectCeleryTaskSignals(TestCase):
    def test_call(self) -> None:
        from celery.signals import (
            after_task_publish,
            before_task_publish,
            task_failure,
            task_prerun,
            task_rejected,
            task_retry,
            task_revoked,
            task_success,
            task_unknown,
        )

        from django_structlog.celery.receivers import CeleryReceiver

        receiver = CeleryReceiver()
        with patch(
            "celery.utils.dispatch.signal.Signal.connect", autospec=True
        ) as mock_connect:
            receiver.connect_worker_signals()

        mock_connect.assert_has_calls(
            [
                call(before_task_publish, receiver.receiver_before_task_publish),
                call(after_task_publish, receiver.receiver_after_task_publish),
                call(task_prerun, receiver.receiver_task_prerun),
                call(task_retry, receiver.receiver_task_retry),
                call(task_success, receiver.receiver_task_success),
                call(task_failure, receiver.receiver_task_failure),
                call(task_revoked, receiver.receiver_task_revoked),
                call(task_unknown, receiver.receiver_task_unknown),
                call(task_rejected, receiver.receiver_task_rejected),
            ]
        )


class TestConnectCelerySignals(TestCase):
    def test_call(self) -> None:
        from celery.signals import after_task_publish, before_task_publish

        from django_structlog.celery.receivers import CeleryReceiver

        receiver = CeleryReceiver()
        with patch(
            "celery.utils.dispatch.signal.Signal.connect", autospec=True
        ) as mock_connect:
            receiver.connect_signals()

        mock_connect.assert_has_calls(
            [
                call(before_task_publish, receiver.receiver_before_task_publish),
                call(after_task_publish, receiver.receiver_after_task_publish),
            ]
        )



import structlog

from .base import *  # noqa: F403
from .base import MIDDLEWARE, env

# GENERAL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#debug
DEBUG = env.bool("DJANGO_DEBUG", default=True)

# https://docs.djangoproject.com/en/dev/ref/settings/#secret-key
SECRET_KEY = env(
    "DJANGO_SECRET_KEY",
    default="DXatocQyyroxzcpo0tDxK3v5Rm4fatD9U7UeuLWwnZMOIaCQdPWovuqp4rxOct1T",
)
# https://docs.djangoproject.com/en/dev/ref/settings/#allowed-hosts
ALLOWED_HOSTS = ["localhost", "0.0.0.0", "127.0.0.1"]

IS_WORKER = env.bool("IS_WORKER", default=False)

# CACHES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#caches
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "",
    }
}

# TEMPLATES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES[0]["OPTIONS"]["debug"] = DEBUG  # noqa F405

# EMAIL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#email-backend
EMAIL_BACKEND = env(
    "DJANGO_EMAIL_BACKEND", default="django.core.mail.backends.console.EmailBackend"
)
# https://docs.djangoproject.com/en/dev/ref/settings/#email-host
EMAIL_HOST = "localhost"
# https://docs.djangoproject.com/en/dev/ref/settings/#email-port
EMAIL_PORT = 1025

INTERNAL_IPS = ["127.0.0.1", "10.0.2.2"]
if env("USE_DOCKER") == "yes":
    import socket

    hostname, _, ips = socket.gethostbyname_ex(socket.gethostname())
    INTERNAL_IPS += [ip[:-1] + "1" for ip in ips]

# django-extensions
# ------------------------------------------------------------------------------
# https://django-extensions.readthedocs.io/en/latest/installation_instructions.html#configuration
INSTALLED_APPS += ["django_extensions"]  # noqa F405
# Celery
# ------------------------------------------------------------------------------

# http://docs.celeryproject.org/en/latest/userguide/configuration.html#task-eager-propagates
CELERY_TASK_EAGER_PROPAGATES = True

CELERY_BEAT_SCHEDULE = {
    "example-scheduled-task": {
        "task": "django_structlog_demo_project.taskapp.celery.scheduled_task",
        "schedule": 30.0,
    },
}

# DATABASES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#databases
DATABASES = {"default": env.db("DATABASE_URL")}

# Your stuff...
# ------------------------------------------------------------------------------

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "json_formatter": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": structlog.processors.JSONRenderer(),
            "foreign_pre_chain": [
                structlog.contextvars.merge_contextvars,
                structlog.processors.TimeStamper(fmt="iso"),
                structlog.stdlib.add_logger_name,
                structlog.stdlib.add_log_level,
                structlog.stdlib.PositionalArgumentsFormatter(),
            ],
        },
        "colored": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": structlog.dev.ConsoleRenderer(colors=True),
            "foreign_pre_chain": [
                structlog.contextvars.merge_contextvars,
                structlog.processors.TimeStamper(fmt="iso"),
                structlog.stdlib.add_logger_name,
                structlog.stdlib.add_log_level,
                structlog.stdlib.PositionalArgumentsFormatter(),
            ],
        },
        "key_value": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": structlog.processors.KeyValueRenderer(
                key_order=["timestamp", "level", "event", "logger"]
            ),
            "foreign_pre_chain": [
                structlog.contextvars.merge_contextvars,
                structlog.processors.TimeStamper(fmt="iso"),
                structlog.stdlib.add_logger_name,
                structlog.stdlib.add_log_level,
                structlog.stdlib.PositionalArgumentsFormatter(),
            ],
        },
    },
    "handlers": {
        "colored_stream": {"class": "logging.StreamHandler", "formatter": "colored"},
        "json_file": {
            "class": "logging.handlers.WatchedFileHandler",
            "filename": "logs/json.log",
            "formatter": "json_formatter",
        },
        "flat_line_file": {
            "class": "logging.handlers.WatchedFileHandler",
            "filename": "logs/flat_line.log",
            "formatter": "key_value",
        },
    },
    "loggers": {
        "django_structlog": {
            "handlers": ["colored_stream", "flat_line_file", "json_file"],
            "level": "INFO",
        },
        "django_structlog_demo_project": {
            "handlers": ["colored_stream", "flat_line_file", "json_file"],
            "level": "INFO",
        },
        "foreign_logger": {
            "handlers": ["colored_stream", "flat_line_file", "json_file"],
            "level": "INFO",
        },
    },
}

structlog.configure(
    processors=[
        structlog.contextvars.merge_contextvars,
        structlog.stdlib.filter_by_level,
        structlog.processors.TimeStamper(fmt="iso"),
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
        structlog.processors.UnicodeDecoder(),
        structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
    ],
    logger_factory=structlog.stdlib.LoggerFactory(),
    cache_logger_on_first_use=True,
)

MIDDLEWARE += [
    "django_structlog.middlewares.RequestMiddleware",
]

DJANGO_STRUCTLOG_CELERY_ENABLED = True
DJANGO_STRUCTLOG_COMMAND_LOGGING_ENABLED = True



import structlog
from django.core import management
from django.core.management import BaseCommand
from django_extensions.management.utils import signalcommand

logger = structlog.getLogger(__name__)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("foo", type=str)

    @signalcommand
    def handle(self, foo, *args, **options):
        logger.info("my log", foo=foo)
        management.call_command("example_nested_command", "buz", verbosity=0)
        logger.info("my log 2")



import pytest

from django_structlog_demo_project.users.adapters import AccountAdapter

pytestmark = pytest.mark.django_db


class TestUserCreationForm:
    def test_account_adapter(self):
        assert AccountAdapter().is_open_for_signup(None)



"""
Base settings to build other settings files upon.
"""

import environ
from django.core.exceptions import ImproperlyConfigured

ROOT_DIR = (
    environ.Path(__file__) - 3
)  # (django_structlog_demo_project/config/settings/base.py - 3 = django_structlog_demo_project/)
APPS_DIR = ROOT_DIR.path("django_structlog_demo_project")

env = environ.Env()

READ_DOT_ENV_FILE = env.bool("DJANGO_READ_DOT_ENV_FILE", default=False)
if READ_DOT_ENV_FILE:
    # OS environment variables take precedence over variables from .env
    env.read_env(str(ROOT_DIR.path(".env")))

# GENERAL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#debug
DEBUG = env.bool("DJANGO_DEBUG", False)
# Local time zone. Choices are
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# though not all of them may be available with every OS.
# In Windows, this must be set to your system time zone.
TIME_ZONE = "UTC"
# https://docs.djangoproject.com/en/dev/ref/settings/#language-code
LANGUAGE_CODE = "en-us"
# https://docs.djangoproject.com/en/dev/ref/settings/#site-id
SITE_ID = 1
# https://docs.djangoproject.com/en/dev/ref/settings/#use-i18n
USE_I18N = True
# https://docs.djangoproject.com/en/dev/ref/settings/#use-tz
USE_TZ = True

# URLS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#root-urlconf
ROOT_URLCONF = "config.urls"
# https://docs.djangoproject.com/en/dev/ref/settings/#wsgi-application
WSGI_APPLICATION = "config.wsgi.application"

# APPS
# ------------------------------------------------------------------------------
DJANGO_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # "django.contrib.humanize", # Handy template tags
    "django.contrib.admin",
]
THIRD_PARTY_APPS = [
    "crispy_forms",
    "crispy_bootstrap5",
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "rest_framework",
]
LOCAL_APPS = [
    "django_structlog_demo_project.command_examples",
    "django_structlog_demo_project.users",
    "django_structlog_demo_project.home",
    # Your stuff: custom apps go here
]
# https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS

# MIGRATIONS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#migration-modules
MIGRATION_MODULES = {"sites": "django_structlog_demo_project.contrib.sites.migrations"}

# AUTHENTICATION
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#authentication-backends
AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
    "allauth.account.auth_backends.AuthenticationBackend",
]
# https://docs.djangoproject.com/en/dev/ref/settings/#auth-user-model
AUTH_USER_MODEL = "users.User"
# https://docs.djangoproject.com/en/dev/ref/settings/#login-redirect-url
LOGIN_REDIRECT_URL = "users:redirect"
# https://docs.djangoproject.com/en/dev/ref/settings/#login-url
LOGIN_URL = "account_login"

# PASSWORDS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#password-hashers
PASSWORD_HASHERS = [
    "django.contrib.auth.hashers.PBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher",
    "django.contrib.auth.hashers.BCryptSHA256PasswordHasher",
    "django.contrib.auth.hashers.BCryptPasswordHasher",
]
# https://docs.djangoproject.com/en/dev/ref/settings/#auth-password-validators
AUTH_PASSWORD_VALIDATORS = []

# MIDDLEWARE
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#middleware
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "allauth.account.middleware.AccountMiddleware",
]

# STATIC
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#static-root
STATIC_ROOT = str(ROOT_DIR("staticfiles"))
# https://docs.djangoproject.com/en/dev/ref/settings/#static-url
if env.bool("ASGI", default=False) or env.bool("WSGI", default=False):
    # asgi will use static files from the runserver_plus
    STATIC_URL = "http://127.0.0.1:8000/static/"
else:
    # runserver_plus serve static files with cors disabled
    STATIC_URL = "/static/"
    # fix cors https://stackoverflow.com/a/66688806/2115513
    from django.contrib.staticfiles import handlers

    # extend StaticFilesHandler to add "Access-Control-Allow-Origin" to every response
    class CORSStaticFilesHandler(handlers.StaticFilesHandler):
        def serve(self, request):
            response = super().serve(request)
            response["Access-Control-Allow-Origin"] = "*"
            return response

    # monkeypatch handlers to use our class instead of the original StaticFilesHandler
    handlers.StaticFilesHandler = CORSStaticFilesHandler
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#std:setting-STATICFILES_DIRS
STATICFILES_DIRS = [str(APPS_DIR.path("static"))]
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

# MEDIA
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = str(APPS_DIR("media"))
# https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = "/media/"

# TEMPLATES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES = [
    {
        # https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        # https://docs.djangoproject.com/en/dev/ref/settings/#template-dirs
        "DIRS": [str(APPS_DIR.path("templates"))],
        "OPTIONS": {
            # https://docs.djangoproject.com/en/dev/ref/settings/#template-debug
            "debug": DEBUG,
            # https://docs.djangoproject.com/en/dev/ref/settings/#template-loaders
            # https://docs.djangoproject.com/en/dev/ref/templates/api/#loader-types
            "loaders": [
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ],
            # https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    }
]
# http://django-crispy-forms.readthedocs.io/en/latest/install.html#template-packs
CRISPY_TEMPLATE_PACK = "bootstrap5"

# FIXTURES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#fixture-dirs
FIXTURE_DIRS = (str(APPS_DIR.path("fixtures")),)

# SECURITY
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#session-cookie-httponly
SESSION_COOKIE_HTTPONLY = True
# https://docs.djangoproject.com/en/dev/ref/settings/#csrf-cookie-httponly
CSRF_COOKIE_HTTPONLY = True
# https://docs.djangoproject.com/en/dev/ref/settings/#secure-browser-xss-filter
SECURE_BROWSER_XSS_FILTER = True
# https://docs.djangoproject.com/en/dev/ref/settings/#x-frame-options
X_FRAME_OPTIONS = "DENY"

# EMAIL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#email-backend
EMAIL_BACKEND = env(
    "DJANGO_EMAIL_BACKEND", default="django.core.mail.backends.smtp.EmailBackend"
)

# ADMIN
# ------------------------------------------------------------------------------
# Django Admin URL.
ADMIN_URL = "admin/"
# https://docs.djangoproject.com/en/dev/ref/settings/#admins
ADMINS = []
# https://docs.djangoproject.com/en/dev/ref/settings/#managers
MANAGERS = ADMINS

# Celery
# ------------------------------------------------------------------------------
INSTALLED_APPS += ["django_structlog_demo_project.taskapp.celery.CeleryAppConfig"]
if USE_TZ:
    # http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-timezone
    CELERY_TIMEZONE = TIME_ZONE
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-broker_url

try:
    CELERY_BROKER_URL = env("CELERY_BROKER_URL")
    CELERY_RESULT_BACKEND = env("CELERY_RESULT_BACKEND")
# This breaks when running tests locally.
except ImproperlyConfigured:
    CELERY_BROKER_URL = "redis://0.0.0.0:6379/0"
    CELERY_RESULT_BACKEND = CELERY_BROKER_URL

BROKER_URL = CELERY_BROKER_URL

# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-accept_content
CELERY_ACCEPT_CONTENT = ["json"]
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-task_serializer
CELERY_TASK_SERIALIZER = "json"
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-result_serializer
CELERY_RESULT_SERIALIZER = "json"
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#task-time-limit
# TODO: set to whatever value is adequate in your circumstances
CELERYD_TASK_TIME_LIMIT = 5 * 60
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#task-soft-time-limit
# TODO: set to whatever value is adequate in your circumstances
CELERYD_TASK_SOFT_TIME_LIMIT = 60
# django-allauth
# ------------------------------------------------------------------------------
ACCOUNT_ALLOW_REGISTRATION = env.bool("DJANGO_ACCOUNT_ALLOW_REGISTRATION", True)
# https://django-allauth.readthedocs.io/en/latest/configuration.html
ACCOUNT_LOGIN_METHODS = {"username"}
# https://django-allauth.readthedocs.io/en/latest/configuration.html
ACCOUNT_SIGNUP_FIELDS = ["email*", "username*", "password1*", "password2*"]
# https://django-allauth.readthedocs.io/en/latest/configuration.html
ACCOUNT_EMAIL_VERIFICATION = "mandatory"
# https://django-allauth.readthedocs.io/en/latest/configuration.html
ACCOUNT_ADAPTER = "django_structlog_demo_project.users.adapters.AccountAdapter"


# Your stuff...
# ------------------------------------------------------------------------------

INSTALLED_APPS += ["django_structlog"]



import pytest
from django.core import management

from django_structlog_demo_project.command_examples.management.commands import (
    example_command,
)

pytestmark = pytest.mark.django_db


class TestCommand:
    def test_command(self):
        assert (
            management.call_command(example_command.Command(), "bar", verbosity=0)
            is None
        )



import logging
import logging.config
import os

import structlog
from celery import Celery, shared_task, signals
from django.apps import AppConfig, apps
from django.conf import settings

from django_structlog.celery.steps import DjangoStructLogInitStep

if not settings.configured:
    # set the default Django settings module for the 'celery' program.
    os.environ.setdefault(
        "DJANGO_SETTINGS_MODULE", "config.settings.local"
    )  # pragma: no cover


app = Celery("django_structlog_demo_project", namespace="CELERY")

app.config_from_object("django.conf:settings")

# A step to initialize django-structlog
app.steps["worker"].add(DjangoStructLogInitStep)


@signals.setup_logging.connect
def receiver_setup_logging(
    loglevel, logfile, format, colorize, **kwargs
):  # pragma: no cover
    logging.config.dictConfig(settings.LOGGING)

    structlog.configure(
        processors=[
            structlog.contextvars.merge_contextvars,
            structlog.stdlib.filter_by_level,
            structlog.processors.TimeStamper(fmt="iso"),
            structlog.stdlib.add_logger_name,
            structlog.stdlib.add_log_level,
            structlog.stdlib.PositionalArgumentsFormatter(),
            structlog.processors.StackInfoRenderer(),
            structlog.processors.format_exc_info,
            structlog.processors.UnicodeDecoder(),
            structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
        ],
        logger_factory=structlog.stdlib.LoggerFactory(),
        cache_logger_on_first_use=True,
    )


class CeleryAppConfig(AppConfig):
    name = "django_structlog_demo_project.taskapp"
    verbose_name = "Celery Config"

    def ready(self):
        installed_apps = [app_config.name for app_config in apps.get_app_configs()]
        app.autodiscover_tasks(lambda: installed_apps, force=True)


@shared_task
def successful_task(foo=None):
    import structlog

    logger = structlog.getLogger(__name__)
    logger.info("This is a successful task")


@shared_task
def failing_task(foo=None, **kwargs):
    raise Exception("This is a failed task")


@shared_task
def nesting_task():
    logger = structlog.getLogger(__name__)
    structlog.contextvars.bind_contextvars(foo="Bar")
    logger.info("This is a nesting task")

    nested_task.delay()


@shared_task
def nested_task():
    logger = structlog.getLogger(__name__)
    logger.info("This is a nested task")


@shared_task
def scheduled_task():
    logger = structlog.getLogger(__name__)
    logger.info("This is a scheduled task")


@shared_task
def rejected_task():
    pass


if not settings.IS_WORKER:  # pragma: no branch

    @shared_task
    def unknown_task():
        """Simulate a task unavailable in the worker for demonstration purpose"""


@signals.before_task_publish.connect
def corrupt_rejected_task(sender=None, headers=None, body=None, **kwargs):
    """Simulate celery's task rejection mechanism by breaking up the message"""
    logger = structlog.getLogger(__name__)
    if headers.get("task") == f"{rejected_task.__module__}.{rejected_task.__name__}":
        logger.warn(
            f"corrupting {rejected_task.__name__}",
            task_id=headers.get("id"),
        )
        del headers["task"]



import pytest

from .. import views

pytestmark = pytest.mark.django_db
pytest_plugins = ("pytest_asyncio",)


class TestEnqueueSuccessfulTask:
    def test(self):
        response = views.enqueue_successful_task(None)
        assert response.status_code == 201


class TestEnqueueFailingTask:
    def test(self):
        response = views.enqueue_failing_task(None)
        assert response.status_code == 201


class TestEnqueueNestingTask:
    def test(self):
        response = views.enqueue_nesting_task(None)
        assert response.status_code == 201


class TestRaiseException:
    def test(self):
        with pytest.raises(Exception) as e:
            views.raise_exception(None)
        assert str(e.value) == "This is a view raising an exception."


class TestLogWithStandardLogger:
    def test(self):
        response = views.log_with_standard_logger(None)
        assert response.status_code == 200


@pytest.mark.asyncio
class TestAsyncView:
    async def test(self, mocker):
        mocker.patch("asyncio.sleep")
        response = await views.async_view(None)
        assert response.status_code == 200


class TestRevokeTask:
    def test(self):
        response = views.revoke_task(None)
        assert response.status_code == 201


class TestEnqueueUnknownTask:
    def test(self):
        response = views.enqueue_unknown_task(None)
        assert response.status_code == 201


class TestEnqueueRejectedTask:
    def test(self):
        response = views.enqueue_rejected_task(None)
        assert response.status_code == 201


@pytest.mark.asyncio
class TestAsyncStreamingViewView:
    async def test(self, mocker):
        response = await views.async_streaming_view(None)
        assert response.status_code == 200

        mocker.patch("asyncio.sleep")
        assert b"0" == await anext(response.streaming_content)
        assert b"1" == await anext(response.streaming_content)
        assert b"2" == await anext(response.streaming_content)
        assert b"3" == await anext(response.streaming_content)
        assert b"4" == await anext(response.streaming_content)

        with pytest.raises(StopAsyncIteration):
            await anext(response.streaming_content)


class TestSyncStreamingViewView:
    def test(self, mocker):
        response = views.sync_streaming_view(None)
        assert response.status_code == 200

        mocker.patch("time.sleep")
        assert b"0" == next(response.streaming_content)
        assert b"1" == next(response.streaming_content)
        assert b"2" == next(response.streaming_content)
        assert b"3" == next(response.streaming_content)
        assert b"4" == next(response.streaming_content)
        with pytest.raises(StopIteration):
            next(response.streaming_content)



from django.urls import re_path

from django_structlog_demo_project.users.views import (
    user_detail_view,
    user_list_view,
    user_redirect_view,
    user_update_view,
)

app_name = "users"
urlpatterns = [
    re_path(r"^$", view=user_list_view, name="list"),
    re_path(r"~redirect/", view=user_redirect_view, name="redirect"),
    re_path(r"~update/", view=user_update_view, name="update"),
    re_path(r"^(?P<username>(\w|\.)+)/", view=user_detail_view, name="detail"),
]



import logging

from django.test import TestCase

from django_structlog import app_settings


class TestAppSettings(TestCase):
    def test_celery_enabled(self) -> None:
        settings = app_settings.AppSettings()

        with self.settings(DJANGO_STRUCTLOG_CELERY_ENABLED=True):
            self.assertTrue(settings.CELERY_ENABLED)

    def test_celery_disabled(self) -> None:
        settings = app_settings.AppSettings()

        with self.settings(DJANGO_STRUCTLOG_CELERY_ENABLED=False):
            self.assertFalse(settings.CELERY_ENABLED)

    def test_request_cancelled_log_level_default(self) -> None:
        settings = app_settings.AppSettings()
        self.assertEqual(settings.REQUEST_CANCELLED_LOG_LEVEL, logging.WARNING)

    def test_request_cancelled_log_level_custom(self) -> None:
        settings = app_settings.AppSettings()

        with self.settings(DJANGO_STRUCTLOG_REQUEST_CANCELLED_LOG_LEVEL=logging.DEBUG):
            self.assertEqual(settings.REQUEST_CANCELLED_LOG_LEVEL, logging.DEBUG)

    def test_status_5xx_log_level_default(self) -> None:
        settings = app_settings.AppSettings()
        self.assertEqual(settings.STATUS_5XX_LOG_LEVEL, logging.ERROR)

    def test_status_5xx_log_level_custom(self) -> None:
        settings = app_settings.AppSettings()

        with self.settings(DJANGO_STRUCTLOG_STATUS_5XX_LOG_LEVEL=logging.CRITICAL):
            self.assertEqual(settings.STATUS_5XX_LOG_LEVEL, logging.CRITICAL)



import structlog
from rest_framework.decorators import api_view
from rest_framework.response import Response

logger = structlog.get_logger(__name__)


@api_view()
def home_api_view(request):
    logger.info("This is a rest-framework structured log")
    return Response({"message": "Hello, world!"})



from unittest.mock import create_autospec, patch

from django.test import TestCase

from django_structlog import apps, commands
from django_structlog.celery import receivers


class TestAppConfig(TestCase):
    def test_celery_enabled(self) -> None:
        app = apps.DjangoStructLogConfig(
            "django_structlog", __import__("django_structlog")
        )
        mock_receiver = create_autospec(spec=receivers.CeleryReceiver)
        with patch(
            "django_structlog.celery.receivers.CeleryReceiver",
            return_value=mock_receiver,
        ):
            with self.settings(DJANGO_STRUCTLOG_CELERY_ENABLED=True):
                app.ready()
        mock_receiver.connect_signals.assert_called_once()

        self.assertTrue(hasattr(app, "_celery_receiver"))
        self.assertIsNotNone(app._celery_receiver)

    def test_celery_disabled(self) -> None:
        app = apps.DjangoStructLogConfig(
            "django_structlog", __import__("django_structlog")
        )

        mock_receiver = create_autospec(spec=receivers.CeleryReceiver)
        with patch(
            "django_structlog.celery.receivers.CeleryReceiver",
            return_value=mock_receiver,
        ):
            with self.settings(DJANGO_STRUCTLOG_CELERY_ENABLED=False):
                app.ready()
        mock_receiver.connect_signals.assert_not_called()

        self.assertFalse(hasattr(app, "_celery_receiver"))

    def test_command_enabled(self) -> None:
        app = apps.DjangoStructLogConfig(
            "django_structlog", __import__("django_structlog")
        )
        mock_receiver = create_autospec(spec=commands.DjangoCommandReceiver)
        with patch(
            "django_structlog.commands.DjangoCommandReceiver",
            return_value=mock_receiver,
        ):
            with self.settings(DJANGO_STRUCTLOG_COMMAND_LOGGING_ENABLED=True):
                app.ready()
        mock_receiver.connect_signals.assert_called_once()

        self.assertTrue(hasattr(app, "_django_command_receiver"))
        self.assertIsNotNone(app._django_command_receiver)

    def test_command_disabled(self) -> None:
        app = apps.DjangoStructLogConfig(
            "django_structlog", __import__("django_structlog")
        )

        mock_receiver = create_autospec(spec=commands.DjangoCommandReceiver)
        with patch(
            "django_structlog.commands.DjangoCommandReceiver",
            return_value=mock_receiver,
        ):
            with self.settings(DJANGO_STRUCTLOG_COMMAND_LOGGING_ENABLED=False):
                app.ready()
        mock_receiver.connect_signals.assert_not_called()

        self.assertFalse(hasattr(app, "_django_command_receiver"))



import django.dispatch

bind_extra_request_metadata = django.dispatch.Signal()
""" Signal to add extra ``structlog`` bindings from ``django``'s request.

:param request: the request returned by the view
:param logger: the logger
:param log_kwargs: dictionary of log metadata for the ``request_started`` event. It contains ``request`` and ``user_agent`` keys. You may modify it to add extra information. 

>>> from django.contrib.sites.shortcuts import get_current_site
>>> from django.dispatch import receiver
>>> from django_structlog import signals
>>> import structlog
>>>
>>> @receiver(signals.bind_extra_request_metadata)
... def bind_domain(request, logger, log_kwargs, **kwargs):
...     current_site = get_current_site(request)
...     structlog.contextvars.bind_contextvars(domain=current_site.domain)

"""

bind_extra_request_finished_metadata = django.dispatch.Signal()
""" Signal to add extra ``structlog`` bindings from ``django``'s finished request and response.

:param logger: the logger
:param response: the response resulting of the request
:param log_kwargs: dictionary of log metadata for the ``request_finished`` event. It contains ``request`` and ``code`` keys. You may modify it to add extra information.

>>> from django.contrib.sites.shortcuts import get_current_site
>>> from django.dispatch import receiver
>>> from django_structlog import signals
>>> import structlog
>>>
>>> @receiver(signals.bind_extra_request_finished_metadata)
... def bind_domain(request, logger, response, log_kwargs, **kwargs):
...     current_site = get_current_site(request)
...     structlog.contextvars.bind_contextvars(domain=current_site.domain)

"""

bind_extra_request_failed_metadata = django.dispatch.Signal()
""" Signal to add extra ``structlog`` bindings from ``django``'s failed request and exception.

:param logger: the logger
:param exception: the exception resulting of the request
:param log_kwargs: dictionary of log metadata for the ``request_failed`` event. It contains ``request`` and ``code`` keys. You may modify it to add extra information.

>>> from django.contrib.sites.shortcuts import get_current_site
>>> from django.dispatch import receiver
>>> from django_structlog import signals
>>> import structlog
>>>
>>> @receiver(signals.bind_extra_request_failed_metadata)
... def bind_domain(request, logger, exception, log_kwargs, **kwargs):
...     current_site = get_current_site(request)
...     structlog.contextvars.bind_contextvars(domain=current_site.domain)

"""

update_failure_response = django.dispatch.Signal()
""" Signal to update response failure response before it is returned.

:param request: the request returned by the view
:param response: the response resulting of the request
:param logger: the logger
:param exception: the exception

>>> from django.dispatch import receiver
>>> from django_structlog import signals
>>> import structlog
>>>
>>> @receiver(signals.update_failure_response)
... def add_request_id_to_error_response(request, response, logger, exception, **kwargs):
...     context = structlog.contextvars.get_merged_contextvars(logger)
...     response['X-Request-ID'] = context["request_id"]

"""
